<?php
// $Id$

/**
 * Public facing pages management.
 */

/**
 * 
 * Determine which css files to load given the current context and return them as an array.
 * 
 *   
 * @return
 * an array of file paths to css files
 */
function theme_hues_css() {
  $css   = array();
  
  if (!variable_get('theme_hues_active', TRUE)) {
    return $css;
  }
  
  // Getting through this loop simply means that some rules MAY be valid according to the given group of contexts.
  $rulesets = theme_hues_rulesets();
  $files    = array();
  foreach ($rulesets as $ruleset) {
    
    $valid = TRUE;
    /*
     * A ruleset will be applied if:
     *   a) the global flag is TRUE for this ruleset
     *   b) all other restrictions resolve as TRUE
     *   c) we're on the theme hues config page
     */
    if (!(arg(0) == 'theme_hues' && arg(1) == 'configure') && (!isset($ruleset['restrictions']['global']) || !_theme_hues_css_global($ruleset['restrictions']['global']))) {
      foreach ($ruleset['restrictions'] as $context => $restriction) {
        if ($context == 'global') {
          continue; // this will fail here, but doesn't invalidate the whole ruleset
        }
        
        $fn = '_theme_hues_css_' . $context;
  
        if (!$fn($restriction)) {
          $valid = FALSE;
          break;
        }
      }
    }
      
    if ($valid) {
      $files[] = _theme_hues_css_ruleset_filename($ruleset);
    }
  }
  
  // the context applies, now check to see if any rules actually have been instantiated
  _theme_hues_css_check_files($css, $files);
  return $css;
}

/**
 * 
 * Create CSS files based on the current rulesets and user defined instances.
 * 
 * @return
 * TRUE on success.
 */
function theme_hues_css_generate_files() {
  theme_hues_css_clear_files();
  drupal_clear_css_cache();
  
  $rulesets = theme_hues_rulesets();  
  foreach ($rulesets as $ruleset) {
    if (!_theme_hues_css_generate_ruleset_file($ruleset)) {
      return FALSE;
    }
  }
  
  return TRUE;
}

/**
 * 
 * Generate a well formatted CSS selector statement from our internal selector data
 * 
 * @param $selector
 * 
 * @return
 * A well formed CSS selector
 */
function theme_hues_css_selector($selector) {
  return join(",", preg_split("/\n+/", $selector['selectors']));
}

/**
 * 
 * Get the current value for a given selector based on the current context.
 * 
 * @param $selector
 * 
 * @return
 * A CSS property value
 */
function theme_hues_css_value($selector) {
  $rulesets  = theme_hues_rulesets(array($selector['rid']));
  $ruleset   = $rulesets[0];
  $instances = theme_hues_selector_instances($selector);  
  
  if (!$ruleset) {
    return "";
  }
  
  if (!$instances) {
    return "";
  }
  
  if ($ruleset['restrictions']['user']['condition'] == 'uid' || $ruleset['restrictions']['user']['condition'] == 'content') {
    /*
     * Find the instance by uid, according to the context and location, ie.
     *   uid: this user's config
     *   content: if the path is node/NID, get the value for the author of the page
     *   content: if the path is theme_hues/configure, get the value for this user
     */
    global $user;
    
    $user_uid = NULL;
    if ($ruleset['restrictions']['user']['condition'] == 'uid') {
      $user_uid = $user->uid;
    }
    elseif(arg(0) == 'node' && (int) arg(1)) {
      $node = node_load(arg(1));
      if ($node) {
        $user_uid = $node->uid;
      }
    }
    elseif(arg(0) == 'theme_hues' && arg(1) == 'configure') {
      $user_uid = $user->uid;
    }

    foreach ($instances as $instance) {
      if ($instance['uid'] == $user_uid) {
        return $instance['value'];
      }
    }
  }
  else {
    if (count($instances) > 1) {
      drupal_set_message(t("Found more than one setting that matches selector instance.  Randomly choosing one."), 'warning');
    }
    
    return $instances[0]['value'];
  }
  
  return "";
}

/**
 * 
 * Get a properly formed CSS instance value from this instance.
 * 
 * @todo 
 * Ensure conformity with the CSS spec.
 * 
 * @param $instance
 * 
 * @return
 * A properly formed CSS value.
 */
function theme_hues_css_instance_value($instance) {
  if (strstr($instance['selector']['property'], 'color') && !preg_match("/^#/", $instance['value'])) {
    return "#" . $instance['value'];
  }
  
  if (strstr($instance['selector']['property'], 'image')) {
    $file = file_load( $instance['value']);
    if (!$file) {
      return;
    }

    $filepath = file_create_url($file->uri);
    return "url('$filepath')";
  }
  
  return $instance['value'];
}

/**
 * 
 * Check context rules against the global rule.
 * 
 * @param $restriction
 * 
 * @return
 * TRUE if value == 1
 */
function _theme_hues_css_global($restriction) {
  return ($restriction['value'] == 1);
}

/**
 * 
 * Check context rules against the current user.
 *
 * @param $restriction
 * 
 * @return
 * Depending on the condition:
 *   - all - always TRUE
 *   - role - current user's roles superimposes value
 *   - content - the current node's author has an instance value stored
 *   - uid  - current user has an instance value stored
 */
function _theme_hues_css_user($restriction) {
  global $user;
  
  if ($restriction['condition'] == 'all') {
    return TRUE;
  }
  
  if ($restriction['condition'] == 'role') {
    foreach (array_keys($user->roles) as $rid) {
      if (in_array($rid, $restriction['value'])) {
        return TRUE;
      }
    }
  }
  
  $test_uid = NULL;
  if ($restriction['condition'] == 'uid') {
    $test_uid = $user->uid;
  }
  
  if ($restriction['condition'] == 'content' && arg(0) == 'node' && (int) arg(1)) {
    $node = node_load(arg(1));
    $test_uid = $node->uid;
  }
  
  if (!$test_uid) {
    return FALSE;
  }
  
  $sql = "
  	SELECT
  		r.rid
  	FROM
  		{theme_hues_ruleset} r
  	INNER JOIN
  		{theme_hues_selector} s
  		ON r.rid = s.rid
  	INNER JOIN
  		{theme_hues_instance} i
  		ON i.sid = s.sid
    WHERE
    	r.rid = :rid
    	AND
    	i.uid = :uid
  ";
  
  
  $has_instance = db_query($sql, array(':rid' => $restriction['rid'], ':uid' => $user->uid))->fetchField();
  return $has_instance;
}

/**
 * 
 * Check context rules against the current node.
 *
 * @param $restriction
 * 
 * @return
 * Depending on the condition:
 *   - none - always TRUE
 *   - type - current node's type superimposes value
 *   - nid  - current node has an instance value stored
 */
function _theme_hues_css_node($restriction) {
  if ($restriction['condition'] == 'none') {
    return TRUE;
  }
  
  if (count($args) < 2 || $args[0] != 'node' || (int) $args[1] == 0) {
    return FALSE;
  }
  
  $node = node_load($args[1]);
  if (!$node) {
    return FALSE;
  }
  
  if ($restriction['condition'] == 'type') {
    if (in_array($node->type, $restriction['value'])) {
      return TRUE;
    }
  }
  
  if ($restriction['condition'] == 'nid') {
    if (in_array($node->nid, $restriction['value'])) {
      return TRUE;
    }
  }
  
  return FALSE;
}

/**
 * 
 * Check context rules against the current path.
 *
 * @param $restriction
 * 
 * @return
 * If this restriction has a value, returns the result of the test against it.
 * No value always returns TRUE.
 */
function _theme_hues_css_path($restriction) {
  if (!$restriction['value']) {
    return TRUE;
  }
  
  $path = arg();
  
  $paths = explode("\n", $restriction['value']);
  $pass  = FALSE;
  foreach ($paths as $test_path) {
    $test = explode("/", $test_path);
    
    $pass = TRUE;
    foreach ($path as $n => $part) {
      if ($test[$n] != '%' && $part != $test[$n]) {
        $pass = FALSE;
        break;
      }
    }

    if ($pass) {
      return TRUE;
    }
  }
  
  return FALSE;
}

/**
 * 
 * Check context rules against the current theme.
 *
 * @param $restriction
 * 
 * @return
 * If this restriction has a value, returns the result of the test against it.
 * No value always returns TRUE.
 */
function _theme_hues_css_theme($restriction) {
  global $theme_key;
  
  if (!$restriction['value']) {
    return TRUE;
  }
  
  $keys = explode("\n", $restriction['value']);
  return (in_array($theme_key, $keys));
}

/**
 * 
 * Check context rules against the current domain.
 *
 * @param $restriction
 * 
 * @return
 * If this restriction has a value, returns the result of the test against it.
 * No value always returns TRUE.
 */
function _theme_hues_css_domain($restriction) {
  if (!$restriction['value']) {
    return TRUE;
  }
  
  $domains = explode("\n", $restriction['value']);
  return (in_array($_SERVER['SERVER_NAME'], $domains));
}

/**
 * CSS file management
 */

/**
 * 
 * Generate a CSS file for a given ruleset.
 * 
 * @param $ruleset
 */
function _theme_hues_css_generate_ruleset_file($ruleset) {
  $filename = _theme_hues_css_ruleset_filename($ruleset);
  
  $instances = theme_hues_instances($ruleset);  
  if (!$instances) {
    return TRUE;
  }
  
  $css       = "
/**
 * This file has been created automatically by the Theme Hues module.
 * Do not edit manually.
 */
 
  ";
  foreach ($instances as $instance) {
    $selectors = theme_hues_css_selector($instance['selector']);
    $value     = theme_hues_css_instance_value($instance);
    $css .= $selectors . "{" . $instance['selector']['property'] . ":" . $value . ";}";
  }
  
  $filepath = file_directory_path() . "/" . variable_get('theme_hues_css_path', 'theme_hues_css');
  if (!file_exists($filepath) && !mkdir($filepath)) {
    drupal_set_message(t("There was an error creating the Theme Hues CSS directory"), 'error');
    return FALSE;
  }
  
  $written = file_put_contents($filepath . "/" . $filename, $css);
  if (!$written) {
    drupal_set_message(t("Error writing CSS data to '!filename'.  Please check permissions.", array('!filename' => $filename)), 'error');
    return FALSE;
  }
 
  return TRUE;
}

/**
 * 
 * Delete all CSS files from our directory.
 */
function theme_hues_css_clear_files() {
  $dir = file_directory_path() . "/" . variable_get("theme_hues_css_path", "theme_hues_css");
  
  $files = glob("$dir/*.css");
  foreach ($files as $file) {
    file_unmanaged_delete("$dir/$files");
  }
}

/**
 * 
 * Create a file name given a ruleset, by concatenating the various contexts in a 
 * specific order and returning the MD5 value.
 * 
 * @param $ruleset
 * 
 * @return
 * A filename (the file may or may not exist)
 */
function _theme_hues_css_ruleset_filename($ruleset) {
  $s = "";
  foreach ($ruleset['restrictions'] as $context => $restriction) {
    $s .= $restriction['context'] . '-' . $restriction['condition'] . '-' . _theme_hues_css_context_token($restriction);
  }
  return md5($s) . '.css';
}

/**
 * 
 * Create a token value from this restriction given the current values in the namespace.
 * 
 * Depending on the context, returns either the context value, or the current corresponding value, ie:
 * 
 *  - user
 *    - uid: the current user uid
 *    - role: the context value
 *    - content: the user uid of the author of the current content
 *    - all: the context value
 *  - node
 *    - nid: the context value
 *    - type: the context value
 *  - path: the context value
 *  - theme: the context value
 *  - domain: the current domain
 *  - global: the keyword 'global'
 *   
 * @param $restriction
 * 
 * @return
 * A token value
 */
function _theme_hues_css_context_token($restriction) {
  if($restriction['context'] == 'user' && ($restriction['condition'] == 'uid' || $restriction['condition'] == 'content')) {
    global $user;
    return $user->uid;
  }

  if ($restriction['context'] == 'domain') {
    return $_SERVER['SERVER_NAME'];
  }
  
  if ($restriction['context'] == 'global') {
    return 'global';
  }
  
  return $restriction['value'];
}

/**
 * 
 * Check for the existance of a list of files.  If they exist, add them to the given array.
 * 
 * @param &$css
 * 
 * @param $files
 * 
 */
function _theme_hues_css_check_files(&$css, $files) {
  $filepath = file_directory_path() . "/" . variable_get('theme_hues_css_path', 'theme_hues_css');
  
  foreach ($files as $file) {
    $fullpath = $filepath . "/" . $file;
    if (file_exists($fullpath)) {
      $css[] = $fullpath;
    }
  }
}

